
;==============================================================
;!Title: VEC3 Macro Lib - Usage Example 2
;!Author: Gustavo J. Fiorenza (aim: gushhfx)
;!Date: 09/01/2009 - Rev: 4
;!
;==============================================================

EnableExplicit
XIncludeFile "../VEC3_MACRO.pbi"
XIncludeFile "FRAMEWORK.pbi"

Global min.VEC3, max.VEC3, drag.VEC3


Structure OBJECT
	position.VEC3 		; endpoint (for visual representation)
	orientation.VEC3	;	object vector
	acceleration.VEC3	; force accumulator
EndStructure


Procedure.i render_object( *player.OBJECT )
	
	Define.f len
	Define.i color
	Define.VEC3 ep, ep2

	VEC3_COPY( *player\orientation, VEC3_TEMP )
	VEC3_TEMP\length = 25
	VEC3_ENDPOINT( VEC3_TEMP, ep2 )
	
	VEC3_ENDPOINT( *player\orientation, ep )														; make an endpoint from the orientation vector
	
	len 	= VEC3_LENGTH( ep ) * 20 																			; this entire mess just to color the line depending on the speed...
	color = RGB( len, 255-len, 0 )																			; ^ ^ ^
	
	VEC3_SCALE( ep, 5.0, ep )																						; just to make it more visible

	Circle( *player\position\x, *player\position\y, 9, $4A4A4A )				; draw the player
	Circle( *player\position\x, *player\position\y, 8, $7F7F7F )				; ^ ^ ^
	Plot( *player\position\x, *player\position\y, $000000 )							; ^ ^ ^
	
	Line( *player\position\x, *player\position\y, ep2\x, ep2\y, #Gray ) ; draw the orientation line.
	Line( *player\position\x, *player\position\y, ep\x, ep\y, color  ) 	; finally, draw the 'force' line.
	
EndProcedure


Procedure.i update_object( *object.OBJECT )

	VEC3_SCALE( *object\acceleration, 0.95, *object\acceleration )							; scale the acceleration vector by something less than 1.0 ( we are easing on the throttle )
	VEC3_ADD( *object\orientation, *object\acceleration, *object\orientation ) 	; add the acceleration vector to our orientation vector
	VEC3_MULTIPLY( *object\orientation, drag, *object\orientation )							; multiply the orientation vector by our drag vector to slow down the body
	VEC3_ENDPOINT( *object\orientation, *object\position )											; finally, calculate the endpoint vector from our orientation vector.

	VEC3_CLAMP( *object\position, min, max, 32 )																; and... keep the player within screen boundaries. try using VEC3_WRAP to shift the player if not.

EndProcedure


Structure PARTICLE Extends OBJECT
	life.f
EndStructure
Global NewList particle.PARTICLE()


Procedure.i update_screen( *player.OBJECT, *enemy.OBJECT )

	ClearScreen( $000000 )

	If StartDrawing( ScreenOutput() )

		render_object( *player )
		render_object( *enemy )
		
		ForEach particle()
		
			update_object( particle() )
			render_object( particle() )
			
			particle()\life - 0.1
			
			If particle()\life <= 0.0
				DeleteElement(particle())
			EndIf
			
		Next
		
		
		StopDrawing()
	EndIf

	FlipBuffers()

EndProcedure


Procedure.i update_input( *player.OBJECT )

	Define.VEC3 throttle, steering

	If ExamineKeyboard()

		; Acceleration, you could make this fully dynamic if you wanted to. Although for simplicity we use a constant value in this case.
		If KeyboardPushed( #PB_Key_Up )
			VEC3_SET( throttle, 0.0, 0.025, 0.0 )
		EndIf
		If KeyboardPushed( #PB_Key_Down )
			VEC3_SET( throttle, 0.0, -0.025, 0.0 )
		EndIf

		; We are going to use a constant steering angle, however you can play with this and make it fully dynamic upon user's input, etc.
		If KeyboardPushed( #PB_Key_Left )
			VEC3_SET( steering, -DEG2RAD( 7.2 ), 0.0, 0.0 )
		EndIf
		If KeyboardPushed( #PB_Key_Right )
			VEC3_SET( steering, DEG2RAD( 7.2 ), 0.0, 0.0 )
		EndIf

		VEC3_ADD( *player\acceleration, throttle, *player\acceleration )
		VEC3_ADD( *player\orientation, steering, *player\orientation )

	EndIf

EndProcedure


;########################


Define.OBJECT player, enemy

VEC3_SET( player\position, #WIDTH / 2 - 100, #HEIGHT / 2 -100, 0.0 ) 	; center point
player\orientation\angle = VEC3_ANGLE_DEG2RAD(-90)										; let the object be oriented upwards

VEC3_SET( enemy\position, #WIDTH / 2, #HEIGHT / 2, 0.0 ) 							; center point
enemy\orientation\angle = VEC3_ANGLE_DEG2RAD(-90)											; let the object be oriented upwards

VEC3_SET( min, 0.0, 0.0 )
VEC3_SET( max, #WIDTH, #HEIGHT )
VEC3_SET( drag, 1.0, 0.95, 1.0 )

Define.i quit

If init()
	Repeat
		
		update_window()
		update_input( player )
		update_object( player )

		enemy\orientation\angle = VEC3_CURVE( VEC3_ANGLE_BETWEEN( player\position, enemy\position )		, enemy\orientation\angle, 25 )
		
		If Random(5) = 1
			If AddElement( particle() )
				
				VEC3_COPY( enemy\position, particle()\position )
				
				particle()\life 								= 5.00 + crandomf()
				particle()\orientation\angle 		= enemy\orientation\angle + (crandomf()*0.2)
				particle()\orientation\length 	= 20.0 + crandomf()
				
			EndIf
		EndIf
		
		update_screen( player, enemy )
		
		Delay(8)
		
	Until quit
EndIf

;########################


; IDE Options = PureBasic 4.30 (Windows - x86)
; CursorPosition = 16
; Folding = ---
; Executable = ..\..\other\misc\vec3_macro_test2.exe
; EnableCompileCount = 944
; EnableBuildCount = 2